#! /usr/bin/perl

$outstem = "tope";
$rsym = 2;

$eol = "\n";	# or "\r\n" for Windows or "\r" for Mac

while($ARGV[0] =~ /^-[or]/) {
    shift, $outstem = shift if $ARGV[0] =~ /^-o/;
    shift, $rsym = shift if $ARGV[0] =~ /^-r/;
}

if(-t STDIN && ! -f $ARGV[0]) {
  print STDERR "Usage:  $0  [-r N] [-o outfilestem]  file.mov
Converts a \"tope\" file (.mov, as read by avn) to a list of Geomview/OOGL .off
files named <outfilestem>000.off, <outfilestem>001.off, ...
Applies N-fold rotational symmetry about Z axis to all faces.
Defaults:  -r 2  -o tope
";
  exit(1);
}


sub getints {
  if(read(ARGV, $_, 4*$_[0]) < 4*$_[0]) {
    print STDERR "Expected $_[0] ints reading $readwhat of tope $tope$eol";
    exit(1);
  }
  unpack("N*", $_);
}

sub getfloats {
  if(!defined($littleendian)) {
    $littleendian = ((unpack("L",pack("N",1)))[0] != 1);
  }

  if(read(ARGV, $_, 4*$_[0]) < 4*$_[0]) {
    print STDERR "Expected $_[0] ints reading $readwhat of tope $tope$eol";
    exit(1);
  }
    
  $_ = pack("L*", unpack("N*", $_)) if($littleendian);

  unpack("f*", $_);
}


$tope = 0;
while(1) {
    while(<>) {
	last if /OFF BINARY/;
    }
    last if $_ eq "";

    ($nv, $nf, $junk) = &getints(3);

    print(STDERR "Unreasonable OFF BINARY header (nv=$nv nf=$nf), tope $tope$eol"), last
	if $nv<=0 || $nf<=0 || $nv>1000000 || $nf>1000000;

    $readwhat = "vertex data for tope $tope";
    @v = &getfloats( 3*$nv );
    @nfv = ();
    @fv0 = ();
    @fv = ();

    for($f = 0; $f < $nf; $f++) {
	push(@fv0, 0+@fv);
	$readwhat = "face $f";
	($count) = &getints( 1 );
	if($count != 3) {
	    print STDERR "Expected a triangle not a ${count}-gon on face $f of tope $tope$eol";
	}
	push(@fv, &getints($count));
	push(@fv, $count);

	($colors) = &getints(1);
	&getints($colors) if $colors;	# skip any colors supplied
    }

    open(STDOUT, sprintf(">%s%03d.off", $outstem, $tope));
    print "OFF\n";
    printf "%d %d %d\n", $rsym*$nv, $rsym*$nf, 0;
    for($r = 0; $r < $rsym; $r++) {
	$c = cos($r*6.28318530718/$rsym);
	$s = sin($r*6.28318530718/$rsym);
	for($i = 0; $i < @v; $i += 3) {
	    printf "%.6g %.6g %.6g%s",
		$v[$i]*$c - $v[$i+1]*$s,
		$v[$i]*$s + $v[$i+1]*$c,
		$v[$i+2], $eol;
	}
    }
    
    print "\n";

    for($r = 0; $r < $rsym; $r++) {
	$vbase = $r*$nv;
	for($f = 0; $f < $nf; $f++) {
	    $fvbase = $fv0[$f];
	    printf "3\t%d %d %d%s",
		$vbase+$fv[$fvbase],
		$vbase+$fv[$fvbase+1],
		$vbase+$fv[$fvbase+2],
		$eol;
	}
	print $eol;
    }
    close(STDOUT);
    printf STDERR "Tope %d  %d vertices, %d faces (*symmetry %d) -> %s%03d.off$eol",
	$tope, $nv, $nf, $rsym, $outstem, $tope;
    $tope++;
}
